// ----------------------------------------------------------------------------
/**
   File: battery.c
  
   System:         DS277x based battery pack
   Language: C
  
   Licensed Material - Property of Marko Pavlin Hardware Pages. No restrictions for 
   members of Electronic society of Slovenia
   
   Uporaba izvorne kode je za lane drutva elektronikov slovenije brez licennih omejitev.
  
   (c) Copyright  Marko Pavlin (C)2009
  
   Address:
   		http://www.feltna.com/hardware
   		E-Mail: development@pavlin.si
  
   Author: Marko Pavlin
   E-Mail (or paypal donations): marko@pavlin.si
  
   Description: Battery gauge and protection DS277x driver
   
   Limitations: Only I2C supported

  
*/  

#ifdef PSOC
/* Following include intended for cypress pSoc API */
	#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#else
	#warning "Implement I2C API functions for your target."
#endif 

#include "battery.h"


/* Calculated battery paramters for EEMB LP7545135 battery, 4500mAh capacity */
const BYTE BAT_EEPROM_LP7545135_4500[BAT_EEPROM_LENGTH] = 
		{
		0x00, 0x1C, 0x20, 0xD7, 0x10, 0x8C, 0x28, 0x06, 0x64, 0x1D, 0x60, 0x10, 0x18, 
		0x39, 0x41, 0x05, 0x08, 0x0B, 0x25, 0x03, 0x04, 0x08, 0x1B, 0x64, 0x00, 0x00,
		0x00, 0x28, 0x14, 0x05 };


/* Calculated battery paramters for EEMB LP755085 battery, 3300mAh capacity */
const BYTE BAT_EEPROM_LP755085_3300[BAT_EEPROM_LENGTH] = 
		{
		0x00, 0x1C, 0x20, 0xD7, 0x10, 0x8C, 0x28, 0x06, 0x64, 0x1D, 0x60, 0x10, 0x18, 
		0x39, 0x41, 0x05, 0x08, 0x0B, 0x25, 0x03, 0x04, 0x08, 0x1B, 0x64, 0x00, 0x00,
		0x00, 0x28, 0x14, 0x05 };




	
/**
 * Battery_Read()
 *
 * Reads single byte from DS277x via I2C
 * 
 * @param[in] 	adr - address
 * @return 		BYTE - byte read
 */
BYTE Battery_Read(BYTE adr)
{
BYTE rv;

	I2Cm_fSendStart(0x59,I2Cm_WRITE);       // Do a write  
	I2Cm_fWrite(adr);                       // Write memory address  
	I2Cm_fSendRepeatStart(0x59,I2Cm_READ);  // Do a read  
	rv = I2Cm_bRead(I2Cm_NAKslave);    		// Read data byte and 
											// NAK the slave to signify  
                                            // end of read.   
    I2Cm_SendStop();                                            
   
    return rv;
}


/**
 * Battery_Write()
 *
 * Writes single byte to DS277x via I2C
 * 
 * @param[in] 	adr - address
 * @param[in] 	val - value
 */
void Battery_Write(BYTE adr, BYTE val)
{

	I2Cm_fSendStart(0x59,I2Cm_WRITE);       // Do a write  
	I2Cm_fWrite(adr);                       // Write memory address
	I2Cm_fWrite(val);                       // Write data
    I2Cm_SendStop();                                            
}



/**
 * Battery_status()
 *
 * Reads DS277x status
 * 
 * @return 	BYTE - status byte
 */
BYTE Battery_status(void)
{
	return Battery_Read(0x01);
}


/**
 * Battery_protection()
 *
 * Reads DS277x battery protection status
 * 
 * @return 	BYTE - protection status byte
 */
 BYTE Battery_protection(void)
{
	return Battery_Read(0x00);
}


/**
 * Battery_voltage()
 *
 * Reads DS277x battery cell voltage
 *
 * @param[in] 	cell - cell number:
 *				cell == 1 - "lower" (closer to GND)
 *				cell == 2 - "upper"
 * @return 	FLOAT - cell voltage in volts
 */
float Battery_voltage(BYTE cell)
{
BYTE addrlo, addrhi;
BYTE lo,hi;
signed int readout;
float volt;

	if ((cell != 1) & (cell != 2)) return 0;
	
	if (cell == 1) 
	{
		addrlo = 0x0d; 
		addrhi = 0x0c;
	}  
	if (cell == 2)
	{
		addrlo = 0x1d; 
		addrhi = 0x1c;
	}  

	lo = Battery_Read( addrlo );
	hi = Battery_Read( addrhi );
	
	readout = (signed int)((hi<<3) 		// strip sign bit & shift to fit 2^9
			  | (lo>>5));
	volt = (float)readout;
	volt *=4.883e-3;
	
	return volt;
}



/**
 * Battery_temperature()
 *
 * Reads DS277x temperature and returns in C
 *
 * @return 	FLOAT - fuel gauge temperature
 */
float Battery_temperature(void)
{
BYTE lo,hi;
signed int readout;
float cels;

	lo = Battery_Read( 0x0b );
	hi = Battery_Read( 0x0a );
	
	readout = (signed int)((hi<<3) 		// strip sign bit & shift to fit 2^9
			  | (lo>>5));
			  
	cels = (float)readout;
	cels *= 0.125 ;		// resolucija 0.125C
	
	return cels;
}


/**
 * Battery_current()
 *
 * Reads DS277x battery pack current
 *
 * @param[in] 	Rsense - sense resistor value in ohms
 * @param[in] 	average - average current flag:
 *				average == 1 - report average current (during last 5.12sec)
 *				average == 0 - report current battery current
 * @return 	FLOAT - battery current   Idischarge<0, Icharge>0
 */
float Battery_current(float Rsense, BYTE average)
{
BYTE lo,hi;
signed int readout;
float amps;

	if (Rsense <= 0) return 0;		// test, Rsense mora biti >0

	if (average == 0)
	{	
		lo = Battery_Read( 0x0f );
		hi = Battery_Read( 0x0e );
	}
	else
	{
		lo = Battery_Read( 0x09 );
		hi = Battery_Read( 0x08 );
	}
	
	readout = (signed int)((hi<<8) | lo);
	amps = (float)readout;
	amps *= (1.5625e-6)/Rsense ;		// resolucija 1.5625uV/Rsense
	
	return amps;
}


/**
 * Battery_acumulated_current()
 *
 * Reads DS277x battery pack acumulated current
 *
 * @param[in] 	Rsense - sense resistor value in ohms
 * @return 	FLOAT - acumulated battery current
 */
float Battery_acumulated_current(float Rsense)
{
BYTE lo,hi;
unsigned int readout;
float amps;

	if (Rsense <= 0) return 0;		// test, Rsense mora biti >0

	lo = Battery_Read( 0x11 );
	hi = Battery_Read( 0x10 );	
	readout = (unsigned int)((hi<<8) | lo);
	amps = (float)readout;
	amps *= (6.26e-6)/Rsense ;		// resolucija 6.25uV/Rsense
	
	return amps;
}



/**
 * Battery_cycles()
 *
 * Reads DS277x battery pack capacity
 *
 * @return 	BYTE - number of charge cycles
 */
BYTE Battery_cycles(void)
{
	return Battery_Read( 0x1e ) ;
}


/**
 * Battery_remaining()
 *
 * Reads DS277x battery pack capacity
 *
 * @param[in] 	RAAC - RAAC flag (if 1, report RAAC, otherwise RSAC
 * @param[in] 	Rsense - sense resistor value in ohms
 * @return 	FLOAT - Remaining capacity in mAh
 */
float Battery_remaining(BYTE RAAC)
{
BYTE lo,hi;
unsigned int readout;
float raac;

  if (RAAC == 1)
  {
	lo = Battery_Read( 0x03 );	// RAAC
	hi = Battery_Read( 0x02 );	
  } else
  {
	lo = Battery_Read( 0x05 ); // RSAC
	hi = Battery_Read( 0x04 );	  
  }
  readout = (unsigned int)((hi<<8) | lo);
  raac = (float)readout;
  raac *= 1.6 ;		// resolucija 1.6mAh
	
  return raac;
}

/**
 * Battery_RARC()
 *
 * Reads DS277x RARC
 *
 * @return 	BYTE - RARC
 */
BYTE Battery_RARC(void)
{
	return Battery_Read( 0x06 ) ;
}

/**
 * Battery_RSRC()
 *
 * Reads DS277x RSRC
 *
 * @return 	BYTE - RSRC
 */
BYTE Battery_RSRC(void)
{
	return Battery_Read( 0x07 ) ;
}

/**
 * Battery_GetVov()
 *
 * Reads DS277x Protction overvoltage value
 *
 * @return 	Overvoltage treshold in Volts
 */
float Battery_GetVov(void)
{
	float vov;
	BYTE rvov;
	
	rvov = Battery_Read( 0x7f ) ;
	vov = (678+2*(float)rvov)*5/1024.0;
	
	return vov;
	
}

/**
 * Battery_SetVov()
 *
 * Sets DS277x Protection overvoltage value
 *
 * @input 	Overvoltage treshold in Volts
 */
void Battery_SetVov(float vov)
{
	BYTE rvov;

	rvov = (BYTE)(0.5 * (vov*1024/5 - 678));
	Battery_Write( 0x7f, rvov ) ;
}



/**
 * Battery_GetVuv()
 *
 * Reads DS277x Protction undervoltage value
 *
 * @return 	Overvoltage treshold value:
 * 	BATTERY_VUV200 = 0,			// 2.00V
	BATTERY_VUV230,				// 2.30V
	BATTERY_VUV245,				// 2.45V
	BATTERY_VUV260,				// 2.60V
 */
BYTE Battery_GetVuv(void)
{
	BYTE vuv;
	
	vuv = Battery_Read( 0x60 ) & 0x0c ;
	vuv >>= 2;
	
	return vuv;
}


/**
 * Battery_SetVuv()
 *
 * Sets DS277x Protection undervoltage value
 *
 * @input 	Undervoltage treshold identifier
 * 	BATTERY_VUV200 = 0,			// 2.00V
	BATTERY_VUV230,				// 2.30V
	BATTERY_VUV245,				// 2.45V
	BATTERY_VUV260,				// 2.60V
 */
void Battery_SetVuv(BYTE vuv)
{
	BYTE rvuv;
	
	rvuv = Battery_Read( 0x60 ) & 0x0c ;
	rvuv |= vuv<<2;
	rvuv |= 0x40;		// Enable undervoltage (UVEN)
	
	Battery_Write( 0x60, rvuv );
}



/**
 * Battery_Init()
 *
 * Writes initial battery setup to shadow RAM and copy this to EEPROM
 * Function is called if Power-On Reset flag is set
 * PORF is cleared after write
 * Other registers are initialized 	
 * @return: battery status after init
 */

BYTE Battery_Init(int BAT_id)
{

	BYTE status, protection, j;	

	I2Cm_Start();
	status = Battery_status();
	protection = Battery_protection();
	if (status & 0x02)
	{	// Power On Reset flag
		Battery_Write(0x69, 100);// sense resistor = 10mOHM
		Battery_SetVov(4.25);
		Battery_SetVuv(BATTERY_VUV245);
		Battery_Write(0x01, 0); // Clear PORF and UVF flag
		for (j=0; j<BAT_EEPROM_LENGTH; j++) 
		{
		
			switch (BAT_id)
			{
				case 	BAT_LP7545135_4500 :
					Battery_Write(BAT_EEPROM_STARTADDRESS+j, BAT_EEPROM_LP7545135_4500[j]); // Write parameters to eeprom
				break;
				
				case 	BAT_LP755085_3300 :
					Battery_Write(BAT_EEPROM_STARTADDRESS+j, BAT_EEPROM_LP755085_3300[j]); // Write parameters to eeprom				
				break;
			}		
			
		}
			
	}
	Battery_Write(0, 0x03); // clear flags
	
	return Battery_status();	
}



/*------------------------------------------------------------------*/
/* End of file                                                      */
/*------------------------------------------------------------------*/

